/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "index/ix.h"
#include "system/sm.h"
#include <tuple>


class AggExecutor : public AbstractExecutor {
   private:
    std::unique_ptr<AbstractExecutor> prev_;
    std::vector<AggregateFunction> aggfunctions;
    std::vector<TabCol> groupby_cols;
    //TODO:灵活key
    //map存储key，value
    // std::vector<std::unique_ptr<RmRecord>> prevrecords;
    std::vector<Agg_Cols> agg_cols;//记录拼接后的数据的顺序
    std::vector<ColMeta> output_cols_;//用于记录对外输出的列
    // bool is_end_;
    std::unique_ptr<RmRecord>  return_rec;
    std::vector<std::unique_ptr<RmRecord>>  return_recs;
    std::vector<ColMeta> prev_cols;
    int rec_len;
    int return_pos;
    std::vector<ColMeta> key_cols;
    std::unordered_map<groupkey*, std::unique_ptr<RmRecord>, CustomHash, CustomEqual> groupagg;
    std::unordered_map<groupkey*, std::unique_ptr<RmRecord>, CustomHash, CustomEqual> havingagg;
    std::unordered_map<groupkey*, std::unique_ptr<RmRecord>, CustomHash, CustomEqual> havingcommon;
    std::shared_ptr<Having> having;
    int key_size;
    bool is_end_;
    int having_rec_size;

   public:
    AggExecutor(std::unique_ptr<AbstractExecutor> prev, std::vector<AggregateFunction> aggfunctions_,std::vector<TabCol> groupby_cols_,std::vector<Agg_Cols> agg_cols_,std::shared_ptr<Having> having_) {
        prev_ = std::move(prev);
        aggfunctions=std::move(aggfunctions_);
        groupby_cols=std::move(groupby_cols_);
        agg_cols=std::move(agg_cols_);
        having=std::move(having_);
        is_end_=false;
        return_pos=0;
        rec_len=0;
        having_rec_size=0;
        //先把之前扫表算子的结果存储起来
        //之前扫表算子的结果可能是空
        
        // for(prev_->beginTuple();!prev_->is_end();prev_->nextTuple()){
        //     auto tuple=prev_->Next();
        //     if(tuple==nullptr){
        //         continue;
        //     }
        //     prevrecords.push_back(std::move(tuple));
        // }
        //计算需要返回的记录的长度
        prev_cols=prev_->cols();
        for(auto &agg_col:agg_cols){
            if(agg_col.is_agg==true&&agg_col.type==ast::AggregateCOUNT){
                if(agg_col.col.col_name=="*"){
                    ColMeta outcol={.tab_name=agg_col.col.tab_name,.name=agg_col.as_name,.type=TYPE_INT,.len=sizeof(int),.offset=rec_len,.index=false};
                    output_cols_.push_back(outcol);
                    rec_len=rec_len+sizeof(int);
                }
                else{
                    for(auto &col:prev_cols){
                        if(agg_col.col.col_name==col.name){
                            ColMeta outcol={.tab_name=agg_col.col.tab_name,.name=agg_col.as_name,.type=TYPE_INT,.len=sizeof(int),.offset=rec_len,.index=false};
                            output_cols_.push_back(outcol);
                            rec_len=rec_len+sizeof(int);
                        }
                    }
                } 
                
            }else{
                for(auto &col:prev_cols){
                    if(agg_col.is_agg==false&&agg_col.col.col_name==col.name){
                        ColMeta outcol={.tab_name=agg_col.col.tab_name,.name=agg_col.col.col_name,.type=col.type,.len=col.len,.offset=rec_len,.index=false};
                        output_cols_.push_back(outcol);
                        rec_len=rec_len+col.len;
                    }
                    if(agg_col.is_agg==true&&agg_col.col.col_name==col.name){
                        ColMeta outcol={.tab_name=agg_col.col.tab_name,.name=agg_col.as_name,.type=col.type,.len=col.len,.offset=rec_len,.index=false};
                        output_cols_.push_back(outcol);
                        rec_len=rec_len+col.len;
                    }
                }
            } 
        }
        for(auto havingagg:having->havingconds){
            if(havingagg.lhs_col.type==ast::AggregateCOUNT){
                having_rec_size=having_rec_size+sizeof(int);
            }else{
                for(auto &col:prev_cols){
                    if(havingagg.lhs_col.agg_col.col_name==col.name){
                        having_rec_size=having_rec_size+col.len;
                    }
                }
            }
        }
    }

    void beginTuple() override {
        //如果前面的算子返回是空的
        //如果没有groupby和having
        if(groupby_cols.empty()){
            //直接开始聚合
            auto new_rec=std::make_unique<RmRecord>(rec_len);
            int offset=0;
            for(auto aggfunction:aggfunctions){
                prev_->beginTuple();
                if(prev_->is_end()){
                    Value empty_value;
                    if(aggfunction.type==ast::AggregateCOUNT){
                        int rec_int=0;
                        empty_value.set_int(rec_int);
                        empty_value.init_raw(sizeof(int));
                        memcpy(new_rec->data+offset,empty_value.raw->data,sizeof(int));
                        offset=offset+sizeof(int);
                    }else{
                        for(auto &col:prev_cols){
                            if(col.name==aggfunction.agg_col.col_name){
                                memset(new_rec->data+offset,'\0',col.len);
                                offset=offset+col.len;
                            }
                        }
                        
                    }
                }else{
                    Value temp=DoAgg(aggfunction);
                    if(temp.type==TYPE_INT){
                        memcpy(new_rec->data+offset,temp.raw->data,sizeof(int));
                        offset=offset+sizeof(int);
                    }else if(temp.type==TYPE_FLOAT){
                        memcpy(new_rec->data+offset,temp.raw->data,sizeof(float));
                        offset=offset+sizeof(float);
                    }else if(temp.type==TYPE_STRING){
                        memcpy(new_rec->data+offset,temp.raw->data,temp.str_val.length());
                        offset=offset+temp.str_val.length();
                    }
                } 
            }
            
            return_recs.push_back(std::move(new_rec));
        }
        else{
            key_size=0;
            //计算groupby的key
            for(auto &groupby_col:groupby_cols){
                for(auto &col:prev_cols){
                    if(groupby_col.col_name==col.name&&col.tab_name==groupby_col.tab_name){
                        key_size=key_size+col.len;
                        key_cols.push_back(col);
                    }
                }
            }
            HashGroupBy();
            // prev_->beginTuple();
            // DoGroupBy();
            // for(auto &pair:groupedrecords){
            //     //先对每一个分组做having操作
            //     if(!having->conds.empty()||!having->havingconds.empty()){
            //         if(!DoHaving(pair.second)){
            //             continue;
            //         } 
            //     }
                    
            //     auto key=pair.first;
            //     auto new_rec=std::make_unique<RmRecord>(rec_len);
            //     int offset=0;
            //     int key_off=0;
            //     int agg_pos=0;
            //     for(auto agg_col:agg_cols){
            //         ColMeta rec_col;
            //         if(agg_col.is_agg==false){
            //             for(auto &col:prev_cols){
            //                 if(agg_col.col.col_name==col.name){
            //                     rec_col=col;
            //                     break;
            //                 }
            //             }
            //             if(rec_col.type==TYPE_INT){
            //                 memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(int));
            //                 key_off=key_off+sizeof(int);
            //                 offset=offset+sizeof(int);
            //             }else if(rec_col.type==TYPE_FLOAT){
            //                 memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(float));
            //                 key_off=key_off+sizeof(float);
            //                 offset=offset+sizeof(float);
            //             }else if(rec_col.type==TYPE_STRING){
            //                 memcpy(new_rec->data+offset,key->key_data+key_off,rec_col.len);
            //                 key_off=key_off+rec_col.len;
            //                 offset=offset+rec_col.len;
            //             }
            //         }else{
            //             Value temp=DoAgg(pair.second,aggfunctions[agg_pos]);
            //             agg_pos++;
            //             if(temp.type==TYPE_INT){
            //                 memcpy(new_rec->data+offset,temp.raw->data,sizeof(int));
            //                 offset=offset+sizeof(int);
            //             }else if(temp.type==TYPE_FLOAT){
            //                 memcpy(new_rec->data+offset,temp.raw->data,sizeof(float));
            //                 offset=offset+sizeof(float);
            //             }else if(temp.type==TYPE_STRING){
            //                 memcpy(new_rec->data+offset,temp.raw->data,temp.str_val.length());
            //                 offset=offset+temp.str_val.length();
            //             }
            //         }
            //     }
            //     return_rec.push_back(std::move(new_rec));
            // }
        }
        //TODO:根据groupby的列进行分组
        //TODO根据having条件进行过滤
    }

    Value DoAgg(AggregateFunction aggfunction){
        //初始化需要返回的record数据
        int offset=0;
            //输出顺序默认是对的,不需要agg_col重新调整
            int rec_int=0;
            float rec_float=0;
            std::string rec_str;
            ColType col_type;
            if(aggfunction.type==ast::AggregateCOUNT)
            {
                int num=0;
                for(prev_->beginTuple();!prev_->is_end();prev_->nextTuple()){
                    num++;
                }
                rec_int=num;
                col_type=TYPE_INT;
            }
            else
            {
                if(aggfunction.type==ast::AggregateMAX){
                    rec_int=INT32_MIN;
                    rec_float=-__FLT_MAX__;
                }else if(aggfunction.type==ast::AggregateMIN){
                    rec_int=INT32_MAX;
                    rec_float=__FLT_MAX__;
                }
                for(;!prev_->is_end();prev_->nextTuple()){
                    auto rec=prev_->Next();
                    for(auto &col:prev_cols){
                        if(col.name==aggfunction.agg_col.col_name){
                            if(col.type==TYPE_INT){
                                col_type=TYPE_INT;
                            }else if(col.type==TYPE_FLOAT){
                                col_type=TYPE_FLOAT;
                            }else if(col.type==TYPE_STRING){
                                col_type=TYPE_STRING;
                            }else{
                                //异常
                            }
                            char * rec_buf=rec->data+col.offset;
                            if(aggfunction.type==ast::AggregateMAX){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec>rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec>rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)>0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunction.type==ast::AggregateMIN){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec<rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec<rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)<0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunction.type==ast::AggregateSUM){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    rec_int=rec_int+cur_rec;
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    rec_float=rec_float+cur_rec;
                                }else if(col.type==TYPE_STRING){
                                    throw InternalError("Unexpected data type");
                                }
                            }
                        }
                    }
                }
            }
            
            Value temp;
            if(col_type==TYPE_INT){
                // std::string rec_str=std::to_string(rec_int);
                temp.set_int(rec_int);
                temp.init_raw(sizeof(int));
            } 
            else if(col_type==TYPE_FLOAT)
            {
                temp.set_float(rec_float);
                temp.init_raw(sizeof(float));
            }else if(col_type==TYPE_STRING){
                temp.set_str(rec_str);
                temp.init_raw(rec_str.length());
            }
            return temp;
    }
    Value DoAgg_withvec(std::vector<std::unique_ptr<RmRecord>>& records,AggregateFunction aggfunction){
        //初始化需要返回的record数据
        int offset=0;
            //输出顺序默认是对的,不需要agg_col重新调整
            int rec_int=0;
            float rec_float=0;
            std::string rec_str;
            ColType col_type;
            if(aggfunction.type==ast::AggregateCOUNT)
            {
                int num=0;
                for(prev_->beginTuple();!prev_->is_end();prev_->nextTuple()){
                    num++;
                }
                rec_int=num;
                col_type=TYPE_INT;
            }
            else
            {
                if(aggfunction.type==ast::AggregateMAX){
                    rec_int=INT32_MIN;
                    rec_float=-__FLT_MAX__;
                }else if(aggfunction.type==ast::AggregateMIN){
                    rec_int=INT32_MAX;
                    rec_float=__FLT_MAX__;
                }
                for(auto& rec:records){
                    // auto rec=prev_->Next();
                    for(auto &col:prev_cols){
                        if(col.name==aggfunction.agg_col.col_name){
                            if(col.type==TYPE_INT){
                                col_type=TYPE_INT;
                            }else if(col.type==TYPE_FLOAT){
                                col_type=TYPE_FLOAT;
                            }else if(col.type==TYPE_STRING){
                                col_type=TYPE_STRING;
                            }else{
                                //异常
                            }
                            char * rec_buf=rec->data+col.offset;
                            if(aggfunction.type==ast::AggregateMAX){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec>rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec>rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)>0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunction.type==ast::AggregateMIN){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec<rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec<rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)<0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunction.type==ast::AggregateSUM){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    rec_int=rec_int+cur_rec;
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    rec_float=rec_float+cur_rec;
                                }else if(col.type==TYPE_STRING){
                                    throw InternalError("Unexpected data type");
                                }
                            }
                        }
                    }
                }
            }
            
            Value temp;
            if(col_type==TYPE_INT){
                // std::string rec_str=std::to_string(rec_int);
                temp.set_int(rec_int);
                temp.init_raw(sizeof(int));
            } 
            else if(col_type==TYPE_FLOAT)
            {
                temp.set_float(rec_float);
                temp.init_raw(sizeof(float));
            }else if(col_type==TYPE_STRING){
                temp.set_str(rec_str);
                temp.init_raw(rec_str.length());
            }
            return temp;
    }

    

    void DoGroupBy(){
        if(prev_->is_end()){
            return;
        }
        std::vector<std::unique_ptr<RmRecord>> group_recs;
        auto prev_rec=prev_->Next();
        auto temp=std::make_unique<RmRecord>(prev_rec->size);
        memcpy(temp->data,prev_rec->data,prev_rec->size);
        group_recs.push_back(std::move(temp));
        //初始化一个key
        //先构造key
        char* key_char=new char[key_size];
        groupkey* key=new groupkey(key_size,key_char);
        int off=0;
        for(auto &key_col:key_cols){
            memcpy(key->key_data+off,prev_rec->data+key_col.offset,key_col.len);
            off=off+key_col.len;
        }
        prev_->nextTuple();
        while(!prev_->is_end()){
            prev_rec=prev_->Next();
            //先构造key
            char* key_char=new char[key_size];
            groupkey* temp_key=new groupkey(key_size,key_char);
            int off=0;
            for(auto &key_col:key_cols){
                memcpy(temp_key->key_data+off,prev_rec->data+key_col.offset,key_col.len);
                off=off+key_col.len;
            }
            //判断与key的一致性
            if(std::memcmp(key->key_data, temp_key->key_data,key->key_len) == 0){
                auto temp=std::make_unique<RmRecord>(prev_rec->size);
                memcpy(temp->data,prev_rec->data,prev_rec->size);
                group_recs.push_back(std::move(temp));
            }else{//分完组了                    
                break;
            }
            prev_->nextTuple();
            
        }
        //先对每一个分组做having操作
        if(!having->conds.empty()||!having->havingconds.empty()){
            if(!DoHaving(group_recs)){
                return;
            } 
        }
        auto new_rec=std::make_unique<RmRecord>(rec_len);
        int offset=0;
        int key_off=0;
        int agg_pos=0;
        for(auto agg_col:agg_cols){
            ColMeta rec_col;
            if(agg_col.is_agg==false){
                for(auto &col:prev_cols){
                    if(agg_col.col.col_name==col.name){
                        rec_col=col;
                        break;
                    }
                }
                if(rec_col.type==TYPE_INT){
                    memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(int));
                    key_off=key_off+sizeof(int);
                    offset=offset+sizeof(int);
                }else if(rec_col.type==TYPE_FLOAT){
                    memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(float));
                    key_off=key_off+sizeof(float);
                    offset=offset+sizeof(float);
                }else if(rec_col.type==TYPE_STRING){
                    memcpy(new_rec->data+offset,key->key_data+key_off,rec_col.len);
                    key_off=key_off+rec_col.len;
                    offset=offset+rec_col.len;
                }
            }else{
                Value temp=DoAgg_withvec(group_recs,aggfunctions[agg_pos]);
                agg_pos++;
                if(temp.type==TYPE_INT){
                    memcpy(new_rec->data+offset,temp.raw->data,sizeof(int));
                    offset=offset+sizeof(int);
                }else if(temp.type==TYPE_FLOAT){
                    memcpy(new_rec->data+offset,temp.raw->data,sizeof(float));
                    offset=offset+sizeof(float);
                }else if(temp.type==TYPE_STRING){
                    memcpy(new_rec->data+offset,temp.raw->data,temp.str_val.length());
                    offset=offset+temp.str_val.length();
                }
            }
        }
        return_rec=std::move(new_rec);
        group_recs.clear();
        return;
    }
    void nextTuple() override {
        return_pos++;
    }

    bool DoHaving(std::vector<std::unique_ptr<RmRecord>>& records){
        //先对普通conds做筛选
        bool is_ok=true;
        is_ok=isequal(prev_cols,having->conds,records[0]);
        if(is_ok==false){
            return false;
        }
        //对聚合做操作
        for(auto &aggcond:having->havingconds){
            Value lhs_val=DoAgg_withvec(records,aggcond.lhs_col);
            //可能存在类型转换？
            if(lhs_val.type==TYPE_FLOAT&&aggcond.rhs_val.type==TYPE_INT){
                float temp=(float)lhs_val.int_val;
                lhs_val.set_float(temp);
            }
            int num;
            if(lhs_val.type==TYPE_INT){
                num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,sizeof(int));
            }else if(lhs_val.type==TYPE_FLOAT){
                num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,sizeof(float));
            }else if(lhs_val.type==TYPE_STRING){
                num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,lhs_val.str_val.length());
            }
            if(aggcond.op==OP_EQ){
                is_ok=(num==0)&&is_ok;
            }else if(aggcond.op==OP_GE){
                is_ok=(num>=0)&&is_ok;
            }else if(aggcond.op==OP_GT){
                is_ok=(num>0)&&is_ok;
            }else if(aggcond.op==OP_LE){
                is_ok=(num<=0)&&is_ok;
            }else if(aggcond.op==OP_LT){
                is_ok=(num<0)&&is_ok;
            }else{
                is_ok=(num!=0)&&is_ok;
            }
            if(is_ok==false){
                return false;
            }
        }
        return is_ok;
    }

    bool isequal(const std::vector<ColMeta> tab_cols,const std::vector<Condition> conds,const std::unique_ptr<RmRecord>& rec){
        //循环比较一个记录的内容与每种where条件
        bool is_equal=true;
        for(auto cond:conds){
            //取出每条记录的列
            auto lhs_col=get_col(tab_cols,cond.lhs_col);
            //取出对应列的data
            char* lhs_data=rec->data+lhs_col->offset;
            ColType lhs_type=lhs_col->type;
            char* rhs_data;
            //取出右值
            if(cond.is_rhs_val){
                //右值把值存在记录里面了
                rhs_data=cond.rhs_val.raw->data;
            }else{
                auto rhs_col=get_col(tab_cols,cond.rhs_col);
                rhs_data=rec->data+rhs_col->offset;
                
            }
            
            //使用ix_compare函数进行对比
            const int col_len=lhs_col->len;
            int num=ix_compare(lhs_data,rhs_data,lhs_type,col_len);
            if(cond.op==OP_EQ){
                is_equal=(num==0)&&is_equal;
            }else if(cond.op==OP_GE){
                is_equal=(num>=0)&&is_equal;
            }else if(cond.op==OP_GT){
                is_equal=(num>0)&&is_equal;
            }else if(cond.op==OP_LE){
                is_equal=(num<=0)&&is_equal;
            }else if(cond.op==OP_LT){
                is_equal=(num<0)&&is_equal;
            }else{
                is_equal=(num!=0)&&is_equal;
            }
            if(is_equal==false){
                return is_equal;
            }
        }
        return is_equal;
    }
    
    void update_hashmap(groupkey* key,std::unique_ptr<RmRecord>& rec){
        auto it=groupagg.find(key);
        bool is_inmap;
        if(it==groupagg.end()){
            is_inmap=false;
        }else{
            is_inmap=true;
        }

        auto new_rec=std::make_unique<RmRecord>(rec_len);
        int offset=0;
        int key_off=0;
        int agg_pos=0;
        for(auto agg_col:agg_cols){
            ColMeta rec_col;
            if(agg_col.is_agg==false){
                auto it=groupagg.find(key);
                    for(auto &col:prev_cols){
                        if(agg_col.col.col_name==col.name&&agg_col.col.tab_name==col.tab_name){
                            rec_col=col;
                            break;
                        }
                    }
                    if(rec_col.type==TYPE_INT){
                        memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(int));
                        key_off=key_off+sizeof(int);
                        offset=offset+sizeof(int);
                    }else if(rec_col.type==TYPE_FLOAT){
                        memcpy(new_rec->data+offset,key->key_data+key_off,sizeof(float));
                        key_off=key_off+sizeof(float);
                        offset=offset+sizeof(float);
                    }else if(rec_col.type==TYPE_STRING){
                        memcpy(new_rec->data+offset,key->key_data+key_off,rec_col.len);
                        key_off=key_off+rec_col.len;
                        offset=offset+rec_col.len;
                    }
            }else{
                int rec_int=0;
                float rec_float=0;
                std::string rec_str;
                ColType col_type;
                if(aggfunctions[agg_pos].type==ast::AggregateCOUNT)
                {
                    int num=0;
                    if(is_inmap==false){
                        
                    }else{
                        char* old_rec=groupagg[key]->data+offset;
                        num=*(int*)old_rec;
                    }
                    num++;
                    rec_int=num;
                    col_type=TYPE_INT;
                }
                else
                {
                    // if(aggfunctions[agg_pos].type==ast::AggregateMAX){
                    //     rec_int=INT32_MIN;
                    //     rec_float=-__FLT_MAX__;
                    // }else if(aggfunctions[agg_pos].type==ast::AggregateMIN){
                    //     rec_int=INT32_MAX;
                    //     rec_float=__FLT_MAX__;
                    // }
                    for(auto &col:prev_cols){
                        if(col.name==aggfunctions[agg_pos].agg_col.col_name&&col.tab_name==aggfunctions[agg_pos].agg_col.tab_name){
                            if(col.type==TYPE_INT){
                                col_type=TYPE_INT;
                                if(is_inmap){
                                    char* old_rec=groupagg[key]->data+offset;
                                    rec_int=*(int*)old_rec;
                                }else{
                                    if(aggfunctions[agg_pos].type==ast::AggregateMAX){
                                        rec_int=INT32_MIN;
                                    }else if(aggfunctions[agg_pos].type==ast::AggregateMIN){
                                        rec_int=INT32_MAX;
                                    }
                                }
                            }else if(col.type==TYPE_FLOAT){
                                col_type=TYPE_FLOAT;
                                if(is_inmap){
                                    char* old_rec=groupagg[key]->data+offset;
                                    rec_float=*(float*)old_rec;
                                }else{
                                    if(aggfunctions[agg_pos].type==ast::AggregateMAX){
                                        rec_float=-__FLT_MAX__;
                                    }else if(aggfunctions[agg_pos].type==ast::AggregateMIN){
                                        rec_float=__FLT_MAX__;
                                    }
                                }
                            }else if(col.type==TYPE_STRING){
                                col_type=TYPE_STRING;
                                if(is_inmap){
                                    char* old_rec=groupagg[key]->data+offset;
                                    rec_str = std::string(old_rec, col.len);
                                }
                                
                            }else{
                                //异常
                            }
                            char* rec_buf=rec->data+col.offset;
                            if(aggfunctions[agg_pos].type==ast::AggregateMAX){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec>rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec>rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)>0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunctions[agg_pos].type==ast::AggregateMIN){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    if(cur_rec<rec_int){
                                        rec_int=cur_rec;
                                    }
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    if(cur_rec<rec_float){
                                        rec_float=cur_rec;
                                    }
                                }else if(col.type==TYPE_STRING){
                                    if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)<0){
                                        rec_str = std::string((char*)rec_buf, col.len);
                                    }
                                }
                            }else if(aggfunctions[agg_pos].type==ast::AggregateSUM){
                                if(col.type==TYPE_INT){
                                    int cur_rec=*(int*)rec_buf;
                                    rec_int=rec_int+cur_rec;
                                }else if(col.type==TYPE_FLOAT){
                                    float cur_rec=*(float*)rec_buf;
                                    rec_float=rec_float+cur_rec;
                                }else if(col.type==TYPE_STRING){
                                    throw InternalError("Unexpected data type");
                                }
                            }
                        }
                    }
                }
                Value temp;
                if(col_type==TYPE_INT){
                    // std::string rec_str=std::to_string(rec_int);
                    temp.set_int(rec_int);
                    temp.init_raw(sizeof(int));
                } 
                else if(col_type==TYPE_FLOAT)
                {
                    temp.set_float(rec_float);
                    temp.init_raw(sizeof(float));
                }else if(col_type==TYPE_STRING){
                    temp.set_str(rec_str);
                    temp.init_raw(rec_str.length());
                }
                if(temp.type==TYPE_INT){
                    memcpy(new_rec->data+offset,temp.raw->data,sizeof(int));
                    offset=offset+sizeof(int);
                }else if(temp.type==TYPE_FLOAT){
                    memcpy(new_rec->data+offset,temp.raw->data,sizeof(float));
                    offset=offset+sizeof(float);
                }else if(temp.type==TYPE_STRING){
                    memcpy(new_rec->data+offset,temp.raw->data,temp.str_val.length());
                    offset=offset+temp.str_val.length();
                }
                agg_pos++;
            }
        }
        groupagg[key]=std::move(new_rec);
    }

    void update_havingmap(groupkey* key,std::unique_ptr<RmRecord>& rec){
        auto it=havingagg.find(key);
        bool is_inmap;
        if(it==havingagg.end()){
            is_inmap=false;
        }else{
            is_inmap=true;
        }

        auto new_rec=std::make_unique<RmRecord>(having_rec_size);
        int offset=0;
        int key_off=0;
        for(auto aggcond:having->havingconds){
            ColMeta rec_col;
            int rec_int=0;
            float rec_float=0;
            std::string rec_str;
            ColType col_type;
            if(aggcond.lhs_col.type==ast::AggregateCOUNT)
            {
                int num=0;
                if(is_inmap==false){
                    
                }else{
                    char* old_rec=havingagg[key]->data+offset;
                    num=*(int*)old_rec;
                }
                num++;
                rec_int=num;
                col_type=TYPE_INT;
            }
            else
            {
                for(auto &col:prev_cols){
                    if(col.name==aggcond.lhs_col.agg_col.col_name&&col.tab_name==aggcond.lhs_col.agg_col.tab_name){
                        // char* old_rec=havingagg[key]->data+offset;
                        if(col.type==TYPE_INT){
                            col_type=TYPE_INT;
                            if(is_inmap){
                                char* old_rec=havingagg[key]->data+offset;
                                rec_int=*(int*)old_rec;
                            }else{
                                if(aggcond.lhs_col.type==ast::AggregateMAX){
                                    rec_int=INT32_MIN;
                                }else if(aggcond.lhs_col.type==ast::AggregateMIN){
                                    rec_int=INT32_MAX;
                                }
                            }
                        }else if(col.type==TYPE_FLOAT){
                            col_type=TYPE_FLOAT;
                            if(is_inmap){
                                char* old_rec=havingagg[key]->data+offset;
                                rec_float=*(float*)old_rec;
                            }else{
                                if(aggcond.lhs_col.type==ast::AggregateMAX){
                                    rec_float=-__FLT_MAX__;
                                }else if(aggcond.lhs_col.type==ast::AggregateMIN){
                                    rec_float=__FLT_MAX__;
                                }
                            }
                        }else if(col.type==TYPE_STRING){
                            col_type=TYPE_STRING;
                            if(is_inmap){
                                char* old_rec=havingagg[key]->data+offset;
                                rec_str = std::string(old_rec, col.len);
                            }
                            
                        }else{
                            //异常
                        }
                        char* rec_buf=rec->data+col.offset;
                        if(aggcond.lhs_col.type==ast::AggregateMAX){
                            if(col.type==TYPE_INT){
                                int cur_rec=*(int*)rec_buf;
                                if(cur_rec>rec_int){
                                    rec_int=cur_rec;
                                }
                            }else if(col.type==TYPE_FLOAT){
                                float cur_rec=*(float*)rec_buf;
                                if(cur_rec>rec_float){
                                    rec_float=cur_rec;
                                }
                            }else if(col.type==TYPE_STRING){
                                if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)>0){
                                    rec_str = std::string((char*)rec_buf, col.len);
                                }
                            }
                        }else if(aggcond.lhs_col.type==ast::AggregateMIN){
                            if(col.type==TYPE_INT){
                                int cur_rec=*(int*)rec_buf;
                                if(cur_rec<rec_int){
                                    rec_int=cur_rec;
                                }
                            }else if(col.type==TYPE_FLOAT){
                                float cur_rec=*(float*)rec_buf;
                                if(cur_rec<rec_float){
                                    rec_float=cur_rec;
                                }
                            }else if(col.type==TYPE_STRING){
                                if(rec_str.empty() or memcmp(rec_buf, rec_str.c_str(), col.len)<0){
                                    rec_str = std::string((char*)rec_buf, col.len);
                                }
                            }
                        }else if(aggcond.lhs_col.type==ast::AggregateSUM){
                            if(col.type==TYPE_INT){
                                int cur_rec=*(int*)rec_buf;
                                rec_int=rec_int+cur_rec;
                            }else if(col.type==TYPE_FLOAT){
                                float cur_rec=*(float*)rec_buf;
                                rec_float=rec_float+cur_rec;
                            }else if(col.type==TYPE_STRING){
                                throw InternalError("Unexpected data type");
                            }
                        }
                    }
                }
            }
            Value temp;
            if(col_type==TYPE_INT){
                // std::string rec_str=std::to_string(rec_int);
                temp.set_int(rec_int);
                temp.init_raw(sizeof(int));
            } 
            else if(col_type==TYPE_FLOAT)
            {
                temp.set_float(rec_float);
                temp.init_raw(sizeof(float));
            }else if(col_type==TYPE_STRING){
                temp.set_str(rec_str);
                temp.init_raw(rec_str.length());
            }
            if(temp.type==TYPE_INT){
                memcpy(new_rec->data+offset,temp.raw->data,sizeof(int));
                offset=offset+sizeof(int);
            }else if(temp.type==TYPE_FLOAT){
                memcpy(new_rec->data+offset,temp.raw->data,sizeof(float));
                offset=offset+sizeof(float);
            }else if(temp.type==TYPE_STRING){
                memcpy(new_rec->data+offset,temp.raw->data,temp.str_val.length());
                offset=offset+temp.str_val.length();
            }
        }
        havingagg[key]=std::move(new_rec);
    }


    void HashGroupBy(){
        for(prev_->beginTuple();!prev_->is_end();prev_->nextTuple()){
            auto prev_rec=prev_->Next();
            char* key_char=new char[key_size];
            groupkey* key=new groupkey(key_size,key_char);
            int off=0;
            for(auto &key_col:key_cols){
                memcpy(key->key_data+off,prev_rec->data+key_col.offset,key_col.len);
                off=off+key_col.len;
            }
            update_hashmap(key,prev_rec);
            //更新having对应的map
            auto it=havingcommon.find(key);
            if(havingcommon.end()==it){
                auto temp=std::make_unique<RmRecord>(prev_rec->size);
                memcpy(temp->data,prev_rec->data,prev_rec->size);
                havingcommon[key]=std::move(temp);
            }
            update_havingmap(key,prev_rec);
        }
        //筛选每个组的having条件
        for(auto& pair:groupagg){
            bool is_ok=true;
            is_ok=isequal(prev_cols,having->conds,havingcommon[pair.first]);
            if(is_ok==false){
                continue;;
            }
            //查看带有聚合的having条件是否满足
            auto& having_rec=havingagg[pair.first];
            int having_rec_off=0;
            for(auto &aggcond:having->havingconds){
                Value lhs_val;
                if(aggcond.lhs_col.type==ast::AggregateCOUNT){
                    char* rec=having_rec->data+having_rec_off;
                    having_rec_off=having_rec_off+sizeof(int);
                    int temp=*(int*)rec;
                    lhs_val.set_int(temp);
                    lhs_val.init_raw(sizeof(int));
                }else{
                    for(auto& col:prev_cols){
                        if(col.name==aggcond.lhs_col.agg_col.col_name&&
                        aggcond.lhs_col.agg_col.tab_name==col.tab_name){
                            char* rec=having_rec->data+having_rec_off;
                            having_rec_off=having_rec_off+col.len;
                            if(col.type==TYPE_INT){
                                int temp=*(int*)rec;
                                lhs_val.set_int(temp);
                                lhs_val.init_raw(col.len);
                            }else if(col.type==TYPE_FLOAT){
                                float temp=*(float*)rec;
                                lhs_val.set_float(temp);
                                lhs_val.init_raw(col.len);
                            }else if(col.type==TYPE_STRING){
                                std::string temp_str=std::string(rec,col.len);
                                lhs_val.set_str(temp_str);
                                lhs_val.init_raw(col.len);
                            }
                        }
                    }
                }
                
                //可能存在类型转换？
                if(lhs_val.type==TYPE_FLOAT&&aggcond.rhs_val.type==TYPE_INT){
                    float temp=(float)lhs_val.int_val;
                    lhs_val.set_float(temp);
                }
                int num;
                if(lhs_val.type==TYPE_INT){
                    num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,sizeof(int));
                }else if(lhs_val.type==TYPE_FLOAT){
                    num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,sizeof(float));
                }else if(lhs_val.type==TYPE_STRING){
                    num=ix_compare(lhs_val.raw->data,aggcond.rhs_val.raw->data,lhs_val.type,lhs_val.str_val.length());
                }
                if(aggcond.op==OP_EQ){
                    is_ok=(num==0)&&is_ok;
                }else if(aggcond.op==OP_GE){
                    is_ok=(num>=0)&&is_ok;
                }else if(aggcond.op==OP_GT){
                    is_ok=(num>0)&&is_ok;
                }else if(aggcond.op==OP_LE){
                    is_ok=(num<=0)&&is_ok;
                }else if(aggcond.op==OP_LT){
                    is_ok=(num<0)&&is_ok;
                }else{
                    is_ok=(num!=0)&&is_ok;
                }
                
            }
            if(is_ok==false){
                continue;
            }else{
                return_recs.push_back(std::move(pair.second));
            }
        }
    }

    std::unique_ptr<RmRecord> Next() override {
        return std::move(return_recs[return_pos]);
    }

    Rid &rid() override { return _abstract_rid; }

    bool is_end() const override { return return_pos>=return_recs.size();};

    const std::vector<ColMeta> &cols() const override{return output_cols_;};
};