#pragma once
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "index/ix.h"
#include "system/sm.h"
#include "storage/disk_manager.h"
class SortMergeJoinExecutor : public AbstractExecutor{
   private:
    std::unique_ptr<DiskManager> disk_manager;
    std::unique_ptr<AbstractExecutor> left_;    // 左儿子节点（需要join的表）
    std::unique_ptr<AbstractExecutor> right_;   // 右儿子节点（需要join的表）
    size_t len_, llen, rlen;                                // join后获得的每条记录的长度
    std::vector<ColMeta> cols_;                 // join后获得的记录的字段
    std::unique_ptr<RmRecord> tuple;
    std::vector<Condition> fed_conds_;          // join条件
    std::vector<TabCol> lcol;  //join条件左值
    std::vector<TabCol> rcol;  //join条件右值
    bool isend;
    //临时文件句柄以及配套的文件中记录的数量
    int tempfd1, tempfd2;
    int left_result, right_result;
    int lfd_rcd_num, rfd_rcd_num;
    //初始归并段长
    int init_segment_len;
    //存放结果的向量和当前空闲位置的标记量
    int result_len;
    //目前的设计是两路归并排序
    //文件句柄和文件名的哈希表
    std::unordered_map<int, std::string> fd2name;
    //设置左右无索引扫描算子结果文件的未输出记录号
    int lrd_point,rrd_point;
    //该文件存放外部排序后的结果
    int lorder_fd, rorder_fd;
    //bool left_need_read, right_need_read;
    std::unique_ptr<RmRecord> lrcd, rrcd;
    bool endflag;
    rcd_segment lrcd_group,rrcd_group;
    int lgroup_num,rgroup_num,llast_num,rlast_num;
    int lfd_point,rfd_point;

    std::vector<std::string> tables;
    std::unordered_map<std::string, int> tablename2fd;
   public:
    SortMergeJoinExecutor(std::unique_ptr<AbstractExecutor> left, std::unique_ptr<AbstractExecutor> right, 
                            std::vector<Condition> conds, std::vector<std::string> tables_) {
        left_ = std::move(left);
        right_ = std::move(right);
        llen = left_->tupleLen();
        rlen = right_->tupleLen();
        len_ = llen + rlen;
        cols_ = left_->cols();
        auto right_cols = right_->cols();
        for (auto &col : right_cols) {
            col.offset += left_->tupleLen();
        }

        cols_.insert(cols_.end(), right_cols.begin(), right_cols.end());
        isend = false;
        fed_conds_ = std::move(conds);
        for(auto it : fed_conds_){
            lcol.push_back(it.lhs_col);
            rcol.push_back(it.rhs_col);
        }
        //设置归并段长
        init_segment_len = 2;
        result_len = 2;
        disk_manager = std::make_unique<DiskManager>();
        lrd_point = 0;
        rrd_point = 0;

        endflag = false;

        tables = std::move(tables_);
    }

    //判断record中的数据是否符合conds中的data的数据,这个扫描不涉及连接查询
    bool compare(const std::vector<ColMeta> tab_cols,const std::vector<TabCol> cols,const std::unique_ptr<RmRecord>& rec1
        , const std::unique_ptr<RmRecord>& rec2){
        //循环比较一个记录的内容与每种where条件
        for(auto col:cols){
            auto col_meta = get_col(tab_cols, col);
            char *data1 = rec1->data + col_meta->offset;
            char *data2 = rec2->data + col_meta->offset;
            if(col_meta->type == TYPE_FLOAT){
                float tempdata1 = *(float *)data1;
                float tempdata2 = *(float *)data2;
                if(tempdata1 != tempdata2){
                    return tempdata1 < tempdata2;
                }
            }
            else if(col_meta->type == TYPE_INT){
                int tempdata1 = *(int *)data1;
                int tempdata2 = *(int *)data2;
                if(tempdata1 != tempdata2){
                    return tempdata1 < tempdata2;
                }
            }
            else if(col_meta->type == TYPE_STRING){
                int ans = memcmp(data1, data2, col_meta->len);
                if(ans<0){
                    return true;
                }
                else if(ans>0){
                    return false;
                }
            }
            else{
                throw InternalError("type1 error");
            }
        }
        return false;
    }
    bool char_compare(const std::vector<ColMeta> tab_cols,const std::vector<TabCol> cols,char *rcd1, char* rcd2){
        for(auto col:cols){
            auto col_meta = get_col(tab_cols, col);
            char * data1 = rcd1 + col_meta->offset;
            char * data2 = rcd2 + col_meta->offset;
            if(col_meta->type == TYPE_FLOAT){
                float tempdata1 = *(float *)data1;
                float tempdata2 = *(float *)data2;
                if(tempdata1 != tempdata2){
                    return tempdata1 < tempdata2;
                }
            }
            else if(col_meta->type == TYPE_INT){
                int tempdata1 = *(int *)data1;
                int tempdata2 = *(int *)data2;
                if(tempdata1 != tempdata2){
                    return tempdata1 < tempdata2;
                }
            }
            else if(col_meta->type == TYPE_STRING){
                int ans = memcmp(data1, data2, col_meta->len);
                if(ans<0){
                    return true;
                }
                else if(ans>0){
                    return false;
                }
            }
            else{
                throw InternalError("type error");
            }
        }
        return false;
    }
    void result_writing(int fd, std::vector<char*> &result, int len){
        char answer[len*result.size()];
        for(int i = 0;i < result.size(); i++){
            memcpy(answer + i * len, result[i], len);
        }
        ssize_t bytes_write = write(fd, answer, len*result.size());
        if(bytes_write != len*result.size()){
            throw UnixError();
        }
    } 

    int merge_sort(std::unique_ptr<AbstractExecutor> &subexe, std::vector<TabCol> tbl, int segment_len){
        std::vector<ColMeta> cols = subexe->cols();
        //设值归并段
        std::vector<std::unique_ptr<RmRecord>> segment1;
        int len = subexe->tupleLen();
        int fd_rcd_num = 0;

        lseek(tempfd1, 0,SEEK_SET);
        for(subexe->beginTuple(); !subexe->is_end(); subexe->nextTuple()){
            segment1.push_back(subexe->Next());
            if(segment1.size() == segment_len){
                std::sort(segment1.begin(),segment1.end(), [this, cols, tbl](std::unique_ptr<RmRecord> &a, std::unique_ptr<RmRecord> &b){
                    return compare(cols, tbl, a, b);
                });
                //将每段排序完成的记录写入文件
                //组合结果，一次性写入
                char answer[len*segment_len];
                for(int i = 0;i < segment_len; i++){
                    memcpy(answer + i * len, segment1[i]->data, len);
                }
                write(tempfd1, answer, len*segment_len);
                fd_rcd_num += segment_len;
                segment1.clear();
            }
        }
        //把segment1中可能剩余的信息写入文件
        if(segment1.size() > 0){
            std::sort(segment1.begin(),segment1.end(), [this, cols, tbl](std::unique_ptr<RmRecord> &a, std::unique_ptr<RmRecord> &b){
                return compare(cols, tbl, a, b);
            });


            char answer[len*segment1.size()];
            for(int i = 0;i < segment1.size(); i++){
                memcpy(answer + i * len, segment1[i]->data, len);
            }
            write(tempfd1, answer, len*segment1.size());

            fd_rcd_num+=segment1.size();
            segment1.clear();
        }

        
        //开始外部排序
        while(fd_rcd_num > segment_len){
            //定位结果文件的文件头，接下来不再重复定位
            lseek(tempfd2, 0, SEEK_SET);
            //计算归并段数量
            int num,group;
            if(fd_rcd_num % segment_len == 0){
                num = fd_rcd_num/segment_len;
            }
            else{
                num = fd_rcd_num/segment_len + 1;
            }
            //计算归并组数量，由于归并段可能是奇数，那么最后一段最后再特殊处理
            group = num/2;
            //开始针对每一组归并
            for(int i = 0;i < group; i++){
                //设置基线以及段内偏移量
                int base1 = segment_len * 2 * i;
                int base2 = base1 + segment_len;
                int flag1 = 0, flag2 =0, right_len;

                //char result[result_len][llen];
                std::vector<char*> result;
                char rcd_group_1[len * init_segment_len], rcd_group_2[len * init_segment_len];
                //初始化两个归并组
                lseek(tempfd1, base1 * len, SEEK_SET);
                read(tempfd1, rcd_group_1, len * init_segment_len);
                flag1 += init_segment_len;
                lseek(tempfd1, base2 * len, SEEK_SET);
                //第二组中可能存在空位，因此每次整段读取的时候都必须检查剩余记录数能否把归并段填满
                if(base2 + init_segment_len > fd_rcd_num){
                    read(tempfd1, rcd_group_2, len * (fd_rcd_num - base2));
                    right_len = fd_rcd_num - base2;
                }
                else{
                    read(tempfd1, rcd_group_2, len * init_segment_len);
                    right_len = init_segment_len;
                }
                flag2 += init_segment_len;
                int seg1_ptr = 0, seg2_ptr = 0;
                while(true){
                    result.push_back(new char[len]);
                    if(char_compare(cols, tbl, rcd_group_1 + seg1_ptr*len, rcd_group_2 + seg2_ptr*len)) {
                        memcpy(result.back(), rcd_group_1 + seg1_ptr*len, len);
                        if(result.size() == result_len){
                            result_writing(tempfd2, result, len);
                            result.clear();
                        }
                        seg1_ptr++;

                        //左边的归并段读完了,接下来，要么读入新的归并段，要不把剩下的元组处理完
                        if(seg1_ptr >= init_segment_len){
                            //此时还可以读入新的归并段
                            if(flag1 < segment_len){
                                lseek(tempfd1, (base1 + flag1) * len, SEEK_SET);
                                read(tempfd1, rcd_group_1, len * init_segment_len);
                                flag1 += init_segment_len;
                                seg1_ptr = 0;
                                continue;
                            }
                            else{
                                //处理result中可能存在的记录
                                if(result.size() > 0){
                                    result_writing(tempfd2, result, len);
                                    result.clear();
                                }
                                
                                write(tempfd2, rcd_group_2+seg2_ptr*len, len*(right_len - seg2_ptr));

                                //处理右边归并段中的记录
                                lseek(tempfd1, (base2 + flag2) * len, SEEK_SET);
                                
                                while(flag2 < segment_len && fd_rcd_num-(base2+flag2) > 0){
                                    if(base2 +flag2 + init_segment_len > fd_rcd_num){
                                        right_len = fd_rcd_num-(base2+flag2);
                                    }
                                    else{
                                        right_len = init_segment_len;
                                    }
                                    read(tempfd1, rcd_group_2, len*right_len);
                                    write(tempfd2, rcd_group_2, len*right_len);
                                    flag2 += init_segment_len;
                                    //seg2_ptr = 0;
                                }
                                
                                break;
                            }
                        }  
                    }
                    else{
                        memcpy(result.back(), rcd_group_2 + seg2_ptr*len, len);
                        if(result.size() == result_len){
                            //lseek(tempfd2, len * )
                            result_writing(tempfd2, result, len);
                            result.clear();
                        }
                        seg2_ptr++;
                        //右边的归并段读完了,接下来，要么读入新的归并段，要不把剩下的元组处理完
                        if(seg2_ptr >= right_len){
                            //判断该段是不是右侧归并段的最后一段
                            if(flag2 < segment_len && fd_rcd_num-(base2+flag2) > 0){
                                lseek(tempfd1, (base2 + flag2) * len, SEEK_SET);
                                if(base2 +flag2 + init_segment_len > fd_rcd_num){
                                    right_len = fd_rcd_num-(base2+flag2);
                                    read(tempfd1, rcd_group_2, len*right_len);
                                }
                                else{
                                    read(tempfd1, rcd_group_2, len * init_segment_len);
                                    right_len = init_segment_len;
                                }
                                flag2 += init_segment_len;
                                seg2_ptr = 0;
                                continue;
                            }
                            else{
                                if(result.size() > 0){
                                    result_writing(tempfd2, result, len);
                                    result.clear();
                                }
                                write(tempfd2, rcd_group_1+seg1_ptr*len, (init_segment_len-seg1_ptr)*len);

                                lseek(tempfd1, (base1 + flag1) * len, SEEK_SET);
                                //char last_rcd[len * init_segment_len];
                                while(flag1 < segment_len){
                                    read(tempfd1, rcd_group_1, len * init_segment_len);
                                    write(tempfd2, rcd_group_1, len * init_segment_len);
                                    flag1 += init_segment_len;
                                }
                                break;
                            }
                        }
                    } 
                }
            }
            //开始处理最后可能独立的归并段
            if(num%2 != 0){
                // for(int i = (num - 1)*segment_len; i < fd_rcd_num; i++){

                // }
                int left_len = (fd_rcd_num - segment_len*2*group)*len;
                char rcd[left_len];
                lseek(tempfd1, segment_len*2*group*len, SEEK_SET);
                read(tempfd1, rcd, left_len);

                write(tempfd2, rcd, left_len);
            }
            //将结果文件的内容写入磁盘
            fsync(tempfd2);
            //交换文件句柄
            int t = tempfd1;
            tempfd1 = tempfd2;
            tempfd2 = t;
            //归并段长翻倍
            segment_len *= 2;
        }
        sleep(5);
        return fd_rcd_num;
    }
    // void output2txt(std::fstream &outfile, int fd, std::vector<ColMeta> cols, int len, int fd_rcd_num){
    //     //首先输出属性信息
    //     outfile << "|";
    //     for(auto col : cols){
    //         outfile << " " << col.name << " |";
    //     }
    //     outfile << "\n";
    //     //输出元组
    //     char rcd[len*result_len];
    //     char *one_rcd;
    //     int loop_num = fd_rcd_num/result_len;
    //     lseek(fd, 0, SEEK_SET);
    //     for(int i = 0; i < loop_num; i++){
            
    //         read(fd, rcd , len*result_len);
    //         for(int j = 0; j < result_len; j++){
    //             one_rcd = rcd + len * j;
    //             outfile << "|";
    //             for (auto &col : cols) {
    //                 std::string col_str;
    //                 char *rec_buf = one_rcd + col.offset;
    //                 if (col.type == TYPE_INT) {
    //                     col_str = std::to_string(*(int *)rec_buf);
    //                 } else if (col.type == TYPE_FLOAT) {
    //                     col_str = std::to_string(*(float *)rec_buf);
    //                 } else if (col.type == TYPE_STRING) {
    //                     col_str = std::string((char *)rec_buf, col.len);
    //                     col_str.resize(strlen(col_str.c_str()));
    //                 }
    //                 outfile << " " << col_str << " |";
    //             }
    //             outfile << "\n";
    //         }
    //     }
    //     //处理最后一个组
    //     int last_rcd = fd_rcd_num % result_len;
    //     read(fd, rcd, last_rcd*len);
    //     for(int j = 0; j < last_rcd; j++){
    //         one_rcd = rcd + len * j;
    //         outfile << "|";
    //         for (auto &col : cols) {
    //             std::string col_str;
    //             char *rec_buf = one_rcd + col.offset;
    //             if (col.type == TYPE_INT) {
    //                 col_str = std::to_string(*(int *)rec_buf);
    //             } else if (col.type == TYPE_FLOAT) {
    //                 col_str = std::to_string(*(float *)rec_buf);
    //             } else if (col.type == TYPE_STRING) {
    //                 col_str = std::string((char *)rec_buf, col.len);
    //                 col_str.resize(strlen(col_str.c_str()));
    //             }
    //             outfile << " " << col_str << " |";
    //         }
    //         outfile << "\n";
    //     }
    // }
    int build_index_scan_resultfile(std::unique_ptr<AbstractExecutor> &subexe, int fd){

        std::vector<std::unique_ptr<RmRecord>> segment1;
        int len = subexe->tupleLen();
        int fd_rcd_num = 0;
        lseek(fd, 0, SEEK_SET);
        for(subexe->beginTuple(); !subexe->is_end(); subexe->nextTuple()){
            segment1.push_back(subexe->Next());
            if(segment1.size() == init_segment_len){
                //将每段排序完成的记录写入文件
                //组合结果，一次性写入
                char answer[len*init_segment_len];
                for(int i = 0;i < init_segment_len; i++){
                    memcpy(answer + i * len, segment1[i]->data, len);
                }
                write(fd, answer, len*init_segment_len);
                fd_rcd_num += init_segment_len;
                segment1.clear();
            }
        }
        //把segment1中可能剩余的信息写入文件
        if(segment1.size() > 0){
            char answer[len*segment1.size()];
            for(int i = 0;i < segment1.size(); i++){
                memcpy(answer + i * len, segment1[i]->data, len);
            }
            write(fd, answer, len*segment1.size());

            fd_rcd_num+=segment1.size();
            segment1.clear();
        }
        return fd_rcd_num;
    }
    std::string getCurrentTimeStr() {
        // 获取当前时间点
        auto now = std::chrono::system_clock::now();
        
        // 转换为系统时间（用于输出）
        std::time_t now_c = std::chrono::system_clock::to_time_t(now);
        std::tm* now_tm = std::localtime(&now_c);
        
        // 获取当前时间的毫秒部分
        auto ms = std::chrono::duration_cast<std::chrono::milliseconds>(now.time_since_epoch()) % 1000;
        
        // 格式化时间为字符串
        char buffer[30];
        snprintf(buffer, sizeof(buffer), "%04d-%02d-%02d %02d:%02d:%02d.%03lld",
                now_tm->tm_year + 1900, now_tm->tm_mon + 1, now_tm->tm_mday,
                now_tm->tm_hour, now_tm->tm_min, now_tm->tm_sec,
                static_cast<long long>(ms.count()));
        
        return std::string(buffer);
    }
    void beginTuple() override {
        //std::fstream outfile;
        //outfile.open("sorted_results.txt", std::ios::out);

        //如果没有索引，需要外部归并排序
        std::string time = getCurrentTimeStr();
        if(!left_->has_index){
            std::string left_tempfd_1 = time+"left_tempfd_1.txt";
            std::string left_tempfd_2 = time+"left_tempfd_2.txt";
            disk_manager->create_file(left_tempfd_1);
            disk_manager->create_file(left_tempfd_2);
            tempfd1 = disk_manager->open_file(left_tempfd_1);
            tempfd2 = disk_manager->open_file(left_tempfd_2);

            fd2name[tempfd1] = left_tempfd_1;
            fd2name[tempfd2] = left_tempfd_2;

            lfd_rcd_num = merge_sort(left_, lcol, init_segment_len);
            lorder_fd = tempfd1;

            //注意，此时tempfd2为需要删除的文件，但是不知道它的文件名
            disk_manager->close_file(tempfd2);
            disk_manager->destroy_file(fd2name.at(tempfd2));
            fd2name.erase(tempfd2);
            //sleep(5);
        }
        else{
            std::string left_index_orderfile = time + "left_index_orderfile.txt";
            disk_manager->create_file(left_index_orderfile);
            lorder_fd = disk_manager->open_file(left_index_orderfile);
            lfd_rcd_num = build_index_scan_resultfile(left_, lorder_fd);
            fd2name[lorder_fd] = left_index_orderfile;
            //sleep(5);
            //left_->beginTuple();
        }
        if(!right_->has_index){
            std::string right_tempfd_1 = time + "right_tempfd_1.txt";
            std::string right_tempfd_2 = time + "right_tempfd_2.txt";
            disk_manager->create_file(right_tempfd_1);
            disk_manager->create_file(right_tempfd_2);
            tempfd1 = disk_manager->open_file(right_tempfd_1);
            tempfd2 = disk_manager->open_file(right_tempfd_2);

            fd2name[tempfd1] = right_tempfd_1;
            fd2name[tempfd2] = right_tempfd_2;

            rfd_rcd_num = merge_sort(right_, rcol, init_segment_len);
            rorder_fd = tempfd1;

            disk_manager->close_file(tempfd2);
            disk_manager->destroy_file(fd2name.at(tempfd2));
            fd2name.erase(tempfd2);
            //sleep(5);
        }
        else{
            std::string right_index_orderfile = time + "right_index_orderfile.txt";
            disk_manager->create_file(right_index_orderfile);
            rorder_fd = disk_manager->open_file(right_index_orderfile);
            rfd_rcd_num = build_index_scan_resultfile(right_, rorder_fd);
            fd2name[rorder_fd] = right_index_orderfile;
            //sleep(5);
            //right_->beginTuple();
        }

        //按照要求顺序输出表
        // assert(tables.size() == 2);
        // if(tables[0].compare(left_->get_table_name()) == 0){
        //     output2txt(outfile, lorder_fd, left_->cols(), llen, lfd_rcd_num);
        //     output2txt(outfile, rorder_fd, right_->cols(), rlen, rfd_rcd_num);
        // }
        // else{
        //     output2txt(outfile, rorder_fd, right_->cols(), rlen, rfd_rcd_num);
        //     output2txt(outfile, lorder_fd, left_->cols(), llen, lfd_rcd_num);
        // }
        // outfile.close();
        
        
        
        if(lfd_rcd_num == 0){
            endflag = true;
            return;
        }
        lseek(lorder_fd, 0, SEEK_SET);
        lgroup_num = lfd_rcd_num/result_len;
        llast_num = lfd_rcd_num%result_len;

        lrcd_group.data = new char[llen*result_len];
        lrcd_group.point = 0;
        if(lgroup_num == 0){
            lrcd_group.len = llast_num;
        }
        else{
            lrcd_group.len = result_len;
        }
        read(lorder_fd, lrcd_group.data, llen*lrcd_group.len);
        lfd_point = 1;
        
        // else{
        //     if(left_->is_end()){
        //         endflag = true;
        //         return;
        //     }
        //     lrcd = left_->Next();
        //     left_->nextTuple();
        // }

        if(rfd_rcd_num == 0){
            endflag = true;
            return;
        }
        lseek(rorder_fd, 0, SEEK_SET);
        rgroup_num = rfd_rcd_num/result_len;
        rlast_num = rfd_rcd_num%result_len;

        rrcd_group.data = new char[rlen*result_len];
        rrcd_group.point = 0;
        if(rgroup_num == 0){
            rrcd_group.len = rlast_num;
        }
        else{
            rrcd_group.len = result_len;
        }
        read(rorder_fd, rrcd_group.data, rlen*rrcd_group.len);
        rfd_point = 1;
        // else{
        //     if(right_->is_end()){
        //         endflag = true;
        //         return;
        //     }
        //     rrcd = right_->Next();
        //     right_->nextTuple();
        // }
    }

    void lfecth_next_rcd(){
        // if(left_->has_index){
        //     if(left_->is_end()){
        //         endflag = true;
        //         return;
        //     }
        //     lrcd = left_->Next();
        //     left_->nextTuple();
        //     return;
        // }
        lrcd_group.point++;
        if(lrcd_group.point < lrcd_group.len){
            return;
        }
        if(lfd_point < lgroup_num){
            lrcd_group.len = result_len;
            read(lorder_fd, lrcd_group.data, llen*lrcd_group.len);
            lrcd_group.point = 0;
            lfd_point++;
        }
        else if(lfd_point == lgroup_num){
            if(rlast_num == 0){
                endflag = true;
                return;
            }
            lrcd_group.len = rlast_num;
            read(lorder_fd, lrcd_group.data, llen*lrcd_group.len);
            lrcd_group.point = 0;
            lfd_point++;
        }
        else{
            endflag = true;
            return;
        }
    }

    void rfecth_next_rcd(){
        // if(right_->has_index){
        //     if(right_->is_end()){
        //         endflag = true;
        //         return;
        //     }
        //     rrcd = right_->Next();
        //     right_->nextTuple();
        //     return;
        // }
        rrcd_group.point++;
        if(rrcd_group.point < rrcd_group.len){
            return;
        }
        if(rfd_point < rgroup_num){
            rrcd_group.len = result_len;
            read(rorder_fd, rrcd_group.data, rlen*rrcd_group.len);
            rrcd_group.point = 0;
            rfd_point++;
        }
        else if(rfd_point == rgroup_num){
            if(rlast_num == 0){
                endflag = true;
                return;
            }
            rrcd_group.len = rlast_num;
            read(rorder_fd, rrcd_group.data, rlen*rrcd_group.len);
            rrcd_group.point = 0;
            rfd_point++;
        }
        else{
            endflag = true;
            return;
        }
    }
    void nextTuple() override {
        return;
    }

    std::unique_ptr<RmRecord> Next() override{

        lrcd = std::make_unique<RmRecord>(llen);
        memcpy(lrcd->data, lrcd_group.data+llen*lrcd_group.point, llen);
    
        rrcd = std::make_unique<RmRecord>(rlen);
        memcpy(rrcd->data, rrcd_group.data+rlen*rrcd_group.point, rlen);

        auto new_record=std::make_unique<RmRecord>(len_);
        memcpy(new_record->data,lrcd->data,lrcd->size);
        memcpy(new_record->data+lrcd->size,rrcd->data,rrcd->size);
        int flag = isequal(cols_,fed_conds_,new_record);
        if(flag == 0){
            lfecth_next_rcd();
            rfecth_next_rcd();
            return new_record;
        }
        else if(flag < 0){
            lfecth_next_rcd();
        }
        else{
            rfecth_next_rcd();
        }
        return nullptr;
    }
    //判断record中的数据是否符合conds中的data的数据,这个扫描不涉及连接查询
    int isequal(const std::vector<ColMeta> tab_cols,const std::vector<Condition> conds,const std::unique_ptr<RmRecord>& rec){
        //循环比较一个记录的内容与每种where条件
        bool is_equal=true;
        for(auto cond:conds){
            //取出每条记录的列
            auto lhs_col=get_col(tab_cols,cond.lhs_col);
            //取出对应列的data
            char* lhs_data=rec->data+lhs_col->offset;
            ColType lhs_type=lhs_col->type;
            char* rhs_data;
            //取出右值
            if(cond.is_rhs_val){
                //右值把值存在记录里面了
                rhs_data=cond.rhs_val.raw->data;
            }else{
                auto rhs_col=get_col(tab_cols,cond.rhs_col);
                rhs_data=rec->data+rhs_col->offset;
                
            }
            
            //使用ix_compare函数进行对比
            const int col_len=lhs_col->len;
            int num=ix_compare(lhs_data,rhs_data,lhs_type,col_len);
            //这里需要修改，等于的时候
            if(num < 0){
                return -1;
            }
            else if(num > 0){
                return 1;
            }
        }
        return 0;
    }
    Rid &rid() override { return _abstract_rid; }

    std::string getType() override { return "sortmerge_join"; };

    bool is_end() const override {
        //在这里删除文件
        if(endflag){
            disk_manager->close_file(lorder_fd); 
            disk_manager->destroy_file(fd2name.at(lorder_fd));
            //fd2name.erase(lorder_fd);

            disk_manager->close_file(rorder_fd); 
            disk_manager->destroy_file(fd2name.at(rorder_fd));
            return true;
        }
        else{
            return false;
        }
    };

    ColMeta get_col_offset(const TabCol &target){
        for(int i=0;i<cols_.size();i++){
            if(cols_[i].name==target.col_name&&cols_[i].tab_name==target.tab_name){
                return cols_[i];
            }
        }
    }

    const std::vector<ColMeta> &cols() const override {
        return cols_;
    }

    size_t tupleLen() const override{ return len_; };

};